import { Service } from "typedi";
import type { CollectionModel } from "../schema/collection.schema";
import type { UserModel } from "../schema/user.schema";
import db from "..";
import collectionLikesTable from "../relations/collection-like.relation";
import { and, eq, inArray } from "drizzle-orm";
import userTable from "../schema/user.schema";
import collectionTable from "../schema/collection.schema";

@Service()
export class collectionLikesRepo {
  // association
  async associateCollectionWithLikes(collectionId: CollectionModel['id'], userIds: UserModel['id'][]): Promise<void> {
    try {
      const existingAssociations = await db
        .select({
          user_id: collectionLikesTable.user_id,
        })
        .from(collectionLikesTable)
        .where(eq(collectionLikesTable.collection_id, collectionId))
        .execute();

      const existingUserIds = existingAssociations.map(row => row.user_id);

      const newUserIds = userIds.filter(userId => !existingUserIds.includes(userId));

      if (newUserIds.length > 0) {
        const valuesToInsert = newUserIds.map(userId => ({
          collection_id: collectionId,
          user_id: userId,
        }));

        await db
          .insert(collectionLikesTable)
          .values(valuesToInsert)
          .onConflictDoNothing()
          .execute();

        console.log(`Collection ${collectionId} associated with new likes: ${newUserIds.join(', ')}`);
      } else {
        console.log(`Ǹo new likes to associate with collection ${collectionId}`);
      }
    } catch (error) {
      console.error("Error associating collection with likes", error);
    }
  }

  // Get likes by collection
  async getLikesbyCollection(collectionId: CollectionModel['id']): Promise<Partial<UserModel>[]> {
    try {
      const likes = await db
        .select({
          id: userTable.id,
          name: userTable.name, // Inclua apenas os campos que são seguros
          // Outros campos que não são sensíveis
        })
        .from(userTable)
        .innerJoin(collectionLikesTable, eq(collectionLikesTable.user_id, userTable.id))
        .where(eq(collectionLikesTable.collection_id, collectionId))
        .execute();

      return likes;
    } catch (error) {
      console.error("Error fetching likes by collection", error);
      throw error;
    }
  }


  async removeLikesFromCollection(collectionId: CollectionModel['id'], userIds: UserModel['id'][]): Promise<void> {
    try {
      await db
        .delete(collectionLikesTable)
        .where(
          and(
            eq(collectionLikesTable.collection_id, collectionId),
            inArray(collectionLikesTable.user_id, userIds)
          )
        )
        .execute();

      console.log(`Likes removed from collection ${collectionId}: ${userIds.join(', ')}`);
    } catch (error) {
      console.error("Error removing likes from collection", error);
      throw error;
    }
  }

  async isAssociationExists(collectionId: CollectionModel['id'], userId: UserModel['id']): Promise<boolean> {
    try {
      const result = await db
        .select()
        .from(collectionLikesTable)
        .where(
          and(
            eq(collectionLikesTable.collection_id, collectionId),
            eq(collectionLikesTable.user_id, userId)
          ))
        .execute();

      return result.length > 0;
    } catch (error) {
      console.error("Error checking if association exists", error);
      throw error;
    }
  }

  async getCollectionsByLike(userId: UserModel['id']): Promise<CollectionModel[]> {
    try {
      const collections = await db
        .select()
        .from(collectionLikesTable)
        .innerJoin(collectionTable, eq(collectionLikesTable.collection_id, collectionTable.id))
        .where(eq(collectionLikesTable.user_id, userId))
        .execute();

      return collections;
    } catch (error) {
      console.error("Error fetching collections by like", error);
      throw error;
    }
  }

  async updateCollectionLikes(collectionId: number, newUserIds: number[]): Promise<void> {
    const existingAssociation = await db
      .select({ user_id: collectionLikesTable.user_id })
      .from(collectionLikesTable)
      .where(eq(collectionLikesTable.collection_id, collectionId))
      .execute();

    const existingUserIds = existingAssociation.map(row => row.user_id);

    const userIdsToAdd = newUserIds.filter(id => !existingUserIds.includes(id));
    const userIdsToRemove = existingUserIds.filter(id => !newUserIds.includes(id));

    if (userIdsToAdd.length > 0) {
      const valuesToInsert = userIdsToAdd.map(userId => ({
        collection_id: collectionId,
        user_id: userId,
      }));

      await db
        .insert(collectionLikesTable)
        .values(valuesToInsert)
        .execute();
    }

    if (userIdsToRemove.length > 0) {
      await db
        .delete(collectionLikesTable)
        .where(
          and(
            eq(collectionLikesTable.collection_id, collectionId),
            inArray(collectionLikesTable.user_id, userIdsToRemove)
          )
        )
        .execute();
    }
  }
}