import { Service } from "typedi";
import type { CollectionModel } from "../schema/collection.schema";
import type { ResourceModel } from "../schema/resource.schema";
import db from "..";
import collectionResourcesTable from "../relations/collection-resource.relation";
import { and, eq, inArray, name } from "drizzle-orm";
import resourceTable from "../schema/resource.schema";
import collectionTable from "../schema/collection.schema";

@Service()
export class collectionResourcesRepo {
  async associateCollectionsWithResources(collectionId: CollectionModel['id'], resourceIds: ResourceModel['id'][]): Promise<void> {
    try {
      const existingAssociations = await db
        .select({
          resource_id: collectionResourcesTable.resource_id,
        })
        .from(collectionResourcesTable)
        .where(eq(collectionResourcesTable.collection_id, collectionId))
        .execute();

        const existingResourceIds = existingAssociations.map(row => row.resource_id);

        const newResourceIds = resourceIds.filter(resourceId => !existingResourceIds.includes(resourceId));

        if (newResourceIds.length > 0) {
          const valuesToInsert = newResourceIds.map(resourceId => ({
            collection_id: collectionId,
            resource_id: resourceId,
          }));

          await db  
            .insert(collectionResourcesTable)
            .values(valuesToInsert)
            .onConflictDoNothing()
            .execute();

          console.log(`Collection ${collectionId} associated with new resources: ${newResourceIds.join(', ')}`);
        } else {
          console.log(`No new resources to associate with collection ${collectionId}`);
        }
    } catch (error) {
      console.error("Error associating collection with resources", error);
    }
  }

  async getResourcesbyCollection(collectionId: CollectionModel['id']): Promise<Partial<ResourceModel>[]> {
    try {
      const resources = await db 
        .select({
          id: resourceTable.id,
          name: resourceTable.name,
        })
        .from(resourceTable)
        .innerJoin(collectionResourcesTable, eq(collectionResourcesTable.resource_id, resourceTable.id))
        .where(eq(collectionResourcesTable.collection_id, collectionId))
        .execute();

      return resources;
    } catch (error) {
      console.error("Error fetching likes by collection", error);
      throw error;
    }
  }

  async removeResourcesFromCollection(collectionId: CollectionModel['id'], resourceIds: ResourceModel['id'][]): Promise<void> {
    try {
      await db
        .delete(collectionResourcesTable)
        .where(
          and(
            eq(collectionResourcesTable.collection_id, collectionId),
            inArray(collectionResourcesTable.resource_id, resourceIds)
          )
        )
        .execute();

      console.log(`Likes removed from collection ${collectionId}: ${resourceIds.join(', ')}`);
    } catch (error) {
      console.error("Error removing resources from collection", error);
      throw error;
    }
  }

  async isAssociationExists(collectionId: CollectionModel['id'], resourceId: ResourceModel['id']): Promise<boolean> {
    try {
      const result = await db
        .select()
        .from(collectionResourcesTable)
        .where(
          and(
            eq(collectionResourcesTable.collection_id, collectionId),
            eq(collectionResourcesTable.resource_id, resourceId)
          )
        )
        .execute();

      return result.length > 0;
    } catch (error) {
      console.error("Error checking if association exists", error);
      throw error;
    }
  }

  async getCollectionsbyResource(resourceId: ResourceModel['id']): Promise<CollectionModel[]> {
    try {
      const collections = await db
        .select({
          id: collectionTable.id,
          name: collectionTable.name,
          description: collectionTable.description,
        })
        .from(collectionResourcesTable)
        .innerJoin(collectionTable, eq(collectionResourcesTable.collection_id, collectionTable.id))
        .where(eq(collectionResourcesTable.resource_id, resourceId))
        .execute();

      return collections;
    } catch (error) {
      console.error("Error fetching collections by resource", error);
      throw error;
    }
  }
}
