import { Service } from "typedi";
import type { ResourceModel } from "../schema/resource.schema";
import type { UserModel } from "../schema/user.schema";
import db from "..";
import { resourceLikesTable, resourceTable, userTable } from "../schema";
import { and, eq, inArray } from "drizzle-orm";

@Service()
export class resourceLikesRepo {
  async associateResourceWithLikes(resourceId: ResourceModel['id'], userIds: UserModel['id'][]): Promise<void> {
    try {
      const existingAssociations = await db
        .select({
          user_id: resourceLikesTable.user_id,
        })
        .from(resourceLikesTable)
        .where(eq(resourceLikesTable.resource_id, resourceId))
        .execute();

      const existingUserIds = existingAssociations.map(row => row.user_id);

      const newUserIds = userIds.filter(userId => !existingUserIds.includes(userId));

      if (newUserIds.length > 0) {
        const valuesToInsert = newUserIds.map(userId => ({
          resource_id: resourceId,
          user_id: userId,
        }));

        await db
          .insert(resourceLikesTable)
          .values(valuesToInsert)
          .onConflictDoNothing()
          .execute();

        console.log(`Resource ${resourceId} associated with new likes: ${newUserIds.join(', ')}`);
      } else {
        console.log(`No new likes to associate with resource ${resourceId}`);
      }
    } catch (error) {
      console.error("Error associating resource with likes", error);
    }
  }

  async getLikesbyResource(resourceId: ResourceModel['id']): Promise<Partial<UserModel>[]> {
    try {
      const likes = await db
        .select({
          user_id: userTable.id,
          user_name: userTable.name, // Include only safe fields
          // Other fields that are not sensitive
        })
        .from(userTable)
        .innerJoin(resourceLikesTable, eq(resourceLikesTable.user_id, userTable.id))
        .where(eq(resourceLikesTable.resource_id, resourceId))
        .execute();

      return likes;
    } catch (error) {
      console.error("Error fetching likes by resource", error);
      throw error;
    }
  }

  async removeLikesFromResource(resourceId: ResourceModel['id'], userIds: UserModel['id'][]): Promise<void> {
    try {
      await db
        .delete(resourceLikesTable)
        .where(
          and(
            eq(resourceLikesTable.resource_id, resourceId),
            inArray(resourceLikesTable.user_id, userIds)
          )
        )
        .execute();

      console.log(`Likes removed from resource ${resourceId}: ${userIds.join(', ')}`);
    } catch (error) {
      console.error("Error removing likes from resource", error);
      throw error;
    }
  }

  async isAssociationExists(resourceId: ResourceModel['id'], userId: UserModel['id']): Promise<boolean> {
    try {
      const result = await db
        .select()
        .from(resourceLikesTable)
        .where(
          and(
            eq(resourceLikesTable.resource_id, resourceId),
            eq(resourceLikesTable.user_id, userId)
          )
        )
        .execute();
      return result.length > 0;
    } catch (error) {
      console.error("Error checking if association exists", error);
      throw error;
    }
  }

  async getResourcesByLike(userId: UserModel['id']): Promise<ResourceModel[]> {
    try {
      const resources = await db
        .select({
          resource_id: resourceTable.id,
          resource_name: resourceTable.name, // Include only safe fields
          // Other fields that are not sensitive
        })
        .from(resourceLikesTable)
        .innerJoin(resourceTable, eq(resourceLikesTable.resource_id, resourceTable.id))
        .where(eq(resourceLikesTable.user_id, userId))
        .execute();

      return resources;
    } catch (error) {
      console.error("Error fetching resources by like", error);
      throw error;
    }
  }
}