import { Service } from "typedi";
import db from "..";
import resourceSubjectsTable from "../relations/resource-subject.relation";
import { and, eq, inArray } from "drizzle-orm";
import type { ResourceModel } from "../schema/resource.schema";
import type { SubjectModel } from "../schema/subject.schema";
import subjectsTable from "../schema/subject.schema";
import resourceTable from "../schema/resource.schema";


@Service()
export class resourceSubjects {
    //associar um recurso a varias disciplinas
    async associateResourceWithSubjects(resourceId: ResourceModel['id'], subjectIds: SubjectModel['id'][]): Promise<void> {
        try {
            // Passo 1: Buscar disciplinas já associadas ao recurso
            const existingAssociations = await db
                .select({
                    subject_id: resourceSubjectsTable.subject_id
                })
                .from(resourceSubjectsTable)
                .where(eq(resourceSubjectsTable.resource_id, resourceId))
                .execute();

            // Extrair os IDs das disciplinas já associadas
            const existingSubjectIds = existingAssociations.map(row => row.subject_id);

            // Passo 2: Filtrar disciplinas que ainda não estão associadas
            const newSubjectIds = subjectIds.filter(subjectId => !existingSubjectIds.includes(subjectId));

            // Passo 3: Associar novas disciplinas (se houver alguma nova, nao deleta as que ja existem)
            if (newSubjectIds.length > 0) {
                const valuesToInsert = newSubjectIds.map(subjectId => ({
                    resource_id: resourceId,
                    subject_id: subjectId
                }));

                await db
                    .insert(resourceSubjectsTable)
                    .values(valuesToInsert)
                    .onConflictDoNothing() // Evita tentativas de duplicação
                    .execute();

                console.log(`Resource ${resourceId} associated with new subjects: ${newSubjectIds.join(", ")}`);
            } else {
                console.log(`No new subjects to associate for resource ${resourceId}`);
            }
        } catch (error) {
            console.error("Error associating resource with subjects:", error);
        }
    }



    async getSubjectsByResource(resourceId: ResourceModel['id']): Promise<SubjectModel[]> {
        try {
            const subjects = await db
                .select()
                .from(subjectsTable)
                .innerJoin(resourceSubjectsTable, eq(resourceSubjectsTable.subject_id, subjectsTable.id))
                .where(eq(resourceSubjectsTable.resource_id, resourceId))
                .execute();
            return subjects.map(row => row.subjects);
        } catch (error) {
            console.error("Error getting subjects by resource:", error);
            return [];
        }
    }



    async removeSubjectFromResource(resourceId: ResourceModel['id'], subjectIds: SubjectModel['id'][]): Promise<void> {
        try {
            await db
                .delete(resourceSubjectsTable)
                .where(
                    and(
                        eq(resourceSubjectsTable.resource_id, resourceId),
                        inArray(resourceSubjectsTable.subject_id, subjectIds
                        )
                    )
                )
                .execute();
            console.log(`Resource ${resourceId} removed from subjects: ${subjectIds.join(", ")}`);
        } catch (error) {
            console.error("Error removing resource from subjects:", error);
        }
    }

    async isAssociationExists(resourceId: ResourceModel['id'], subjectId: SubjectModel['id']): Promise<boolean> {
        try {
            const association = await db
                .select()
                .from(resourceSubjectsTable)
                .where(
                    and(
                        eq(resourceSubjectsTable.resource_id, resourceId),
                        eq(resourceSubjectsTable.subject_id, subjectId)
                    )
                )
                .execute();
            return association.length > 0;
        } catch (error) {
            console.error("Error checking association:", error);
            throw error;
        }
    }


    async getResourcesBySubject(subjectId: SubjectModel['id']): Promise<ResourceModel[]> {
        try {
            const resources = await db
                .select()
                .from(resourceSubjectsTable)
                .innerJoin(resourceTable, eq(resourceTable.id, resourceSubjectsTable.resource_id))
                .where(eq(resourceSubjectsTable.subject_id, subjectId))
                .execute();
            return resources.map(row => row.resource);
        } catch (error) {
            console.error("Error getting resources by subject:", error);
            return [];
        }
    }

    async updateResourceSubjects(resourceId: ResourceModel['id'], subjectIds: SubjectModel['id'][]): Promise<void> {

        const existingAssociations = await db
            .select({
                subject_id: resourceSubjectsTable.subject_id
            })
            .from(resourceSubjectsTable)
            .where(eq(resourceSubjectsTable.resource_id, resourceId))
            .execute();

        const existingSubjectIds = existingAssociations.map(row => row.subject_id);
        const subjectsToAdd = subjectIds.filter(subjectId => !existingSubjectIds.includes(subjectId));
        const subjectsToRemove = existingSubjectIds.filter(subjectId => !subjectIds.includes(subjectId));

        if (subjectsToAdd.length > 0) {
            const valuesToInsert = subjectsToAdd.map(subjectId => ({
                resource_id: resourceId,
                subject_id: subjectId
            }));

            await db
                .insert(resourceSubjectsTable)
                .values(valuesToInsert)
                .execute();

            console.log(`Resource ${resourceId} associated with new subjects: ${subjectsToAdd.join(", ")}`);
        }

        if (subjectsToRemove.length > 0) {
            await db
                .delete(resourceSubjectsTable)
                .where(
                    and(
                        eq(resourceSubjectsTable.resource_id, resourceId),
                        inArray(resourceSubjectsTable.subject_id, subjectsToRemove)
                    )
                )
                .execute();

            console.log(`Resource ${resourceId} removed from subjects: ${subjectsToRemove.join(", ")}`);
        }
    }
}