import { Service } from "typedi";
import type { UserModel } from "../schema/user.schema";
import db from "..";
import { achievementTable, userAchievementsTable, userTable } from "../schema";
import { and, eq, inArray } from "drizzle-orm";
import type { AchievementModel } from "../schema/achievement.schema";

@Service()
export class userAchievementsRepo {
  async associateUserWithAchievements(userId: UserModel['id'], achievementIds: UserModel['id'][]): Promise<void> {
    try {
      const existingAssociations = await db
        .select({
          user_id: userAchievementsTable.user_id,
        })
        .from(userAchievementsTable)
        .where(eq(userAchievementsTable.user_id, userId))
        .execute();

      const existingAchievementIds = existingAssociations.map(row => row.user_id);

      const newAchievementIds = achievementIds.filter(achievementId => !existingAchievementIds.includes(achievementId));

      if (newAchievementIds.length > 0) {
        const valuesToInsert = newAchievementIds.map(achievementId => ({
          user_id: userId,
          achievement_id: achievementId,
        }));

        await db
          .insert(userAchievementsTable)
          .values(valuesToInsert)
          .onConflictDoNothing()
          .execute();

        console.log(`User ${userId} associated with new achievements: ${newAchievementIds.join(', ')}`);
      } else {
        console.log(`No new achievements to associate with user ${userId}`);
      }
    } catch (error) {
      console.error("Error associating user with achievements", error);
    }
  }

  async getAchievementsByUser(userId: UserModel['id']): Promise<Partial<AchievementModel>[]> {
    try {
      const achievements = await db
        .select({
          id: achievementTable.id,
          name: achievementTable.name,
        })
        .from(achievementTable)
        .innerJoin(userAchievementsTable, eq(userAchievementsTable.achievement_id, achievementTable.id))
        .where(eq(userAchievementsTable.user_id, userId))
        .execute();

      return achievements;
    } catch (error) {
      console.error("Error getting achievements by user", error);
      throw error;
    }
  }

  async removeAchievementsFromUser(userId: UserModel['id'], achievementIds: UserModel['id'][]): Promise<void> {
    try {
      await db
        .delete(userAchievementsTable)
        .where(
          and(
            eq(userAchievementsTable.user_id, userId),
            inArray(userAchievementsTable.achievement_id, achievementIds)
          )
        )
        .execute();

      console.log(`Achievements ${achievementIds.join(', ')} removed from user ${userId}`);
    } catch (error) {
      console.error("Error removing achievements from user", error);
      throw error;
    }
  }

  async isAssociationExists(userId: UserModel['id'], achievementId: AchievementModel['id']): Promise<boolean> {
    try {
      const result = await db
        .select()
        .from(userAchievementsTable)
        .where(
          and(
            eq(userAchievementsTable.user_id, userId),
            eq(userAchievementsTable.achievement_id, achievementId)
          )
        )
        .execute();

      return result.length > 0;
    } catch (error) {
      console.error("Error checking if association exists", error);
      throw error;
    }
  }

  async getUsersByAchievement(achievementId: AchievementModel['id']): Promise<UserModel[]> {
    try {
      const users = await db
        .select({
          id: userTable.id,
          name: userTable.name,
        })
        .from(userAchievementsTable)
        .innerJoin(userTable, eq(userAchievementsTable.user_id, userTable.id))
        .where(eq(userAchievementsTable.achievement_id, achievementId))
        .execute();

      return users;
    } catch (error) {
      console.error("Error getting users by achievement", error);
      throw error;
    }
  }

  async updateUserAchievements(userId: number, newAchievementIds: number[]): Promise<void> {
    const existingAssociation = await db
      .select({ achievementTable_id: userAchievementsTable.achievement_id })
      .from(userAchievementsTable)
      .where(eq(userAchievementsTable.user_id, userId))
      .execute();

    const existingAchievementIds = existingAssociation.map(row => row.achievementTable_id);

    const achievementsIdsToAdd = newAchievementIds.filter(id => !existingAchievementIds.includes(id));
    const achievementsIdsToRemove = existingAchievementIds.filter(id => !newAchievementIds.includes(id));

    if (achievementsIdsToAdd.length > 0) {
      const valuesToInsert = achievementsIdsToAdd.map(achievementId => ({
        user_id: userId,
        achievementId: achievementId,
      }));

      await db
        .insert(userAchievementsTable)
        .values(valuesToInsert)
        .execute();
    }

    if (achievementsIdsToRemove.length > 0) {
      await db
        .delete(userAchievementsTable)
        .where(
          and(
            eq(userAchievementsTable.user_id, userId),
            inArray(userAchievementsTable.achievement_id, achievementsIdsToRemove)
          )
        )
        .execute();
    }
  }
}