import {
  boolean,
  integer,
  pgTable,
  serial, 
  text, 
  timestamp, 
  varchar,
} from 'drizzle-orm/pg-core'
import { createInsertSchema, createSelectSchema } from 'drizzle-zod'
import { z } from 'zod'
import userTable from './user.schema'
import collectionStatsTable from './collection-stats.schema'
import { sql } from 'drizzle-orm'

const collectionTable = pgTable('collections', {
  id: serial('id').primaryKey()
    .notNull()
    .unique(),
  name: varchar('name', { length: 255 }),
  description: text('description'),
  is_private: boolean('is_private')
    .default(false),
  is_active: boolean('is_active')
    .default(true),
  created_at: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updated_at: timestamp('updated_at', { mode: 'string' })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),
  deleted_at: timestamp('deleted_at', { mode: 'string' }),
  thumbnail: varchar('thumbnail', { length: 255 }),
  user_id: integer('user_id')
    .notNull()
    .references(() => userTable.id, {onDelete: 'cascade'}),
  collection_stats_id: integer('collection_stats_id')
    .notNull()
    .references(() => collectionStatsTable.id, {onDelete: 'cascade'}),
})

const collectionModelSchema = createSelectSchema(collectionTable)
const collectionDtoSchema = collectionModelSchema
const collectionInputSchema = createInsertSchema(collectionTable)
.partial()
.required({user_id: true})
const collectionUpdateSchema = collectionInputSchema.partial().required({ id: true })

export type CollectionModel = z.infer<typeof collectionModelSchema>
export type CollectionDto = z.infer<typeof collectionDtoSchema>
export type CollectionInput = z.infer<typeof collectionInputSchema>
export type CollectionUpdate = z.infer<typeof collectionUpdateSchema>

export const collectionSchemas = {
  collectionModelSchema,
  collectionDtoSchema,
  collectionInputSchema,
  collectionUpdateSchema,
}

export default collectionTable