import { boolean, serial, text, timestamp, varchar } from "drizzle-orm/pg-core";
import { integer, numeric, pgTable } from "drizzle-orm/pg-core";
import achievementTable from "./achievement.schema";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";
import { sql } from "drizzle-orm";



const itemsTable = pgTable('items', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  name: varchar('name', { length: 255 })
    .notNull(),
  price: numeric('price'),
  discount: numeric('discount'),
  description: text('description'),
  is_active: boolean('is_active')
    .default(false),
  created_at: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updated_at: timestamp('updated_at', { mode: 'string' })
  .notNull()
  .defaultNow()
  .$onUpdate(() => sql`current_timestamp`),
  achievement_id: integer('achievement_id')
    .references(() => achievementTable.id)
})

const itemsModelSchema = createSelectSchema(itemsTable)
const itemsDtoSchema = itemsModelSchema.omit({})
const itemsInputSchema = createInsertSchema(itemsTable)
const itemsUpdateSchema = itemsInputSchema
    .partial()
    .required({ id: true })

export type ItemsModel = z.infer<typeof itemsModelSchema>
export type ItemsDto = z.infer<typeof itemsDtoSchema>
export type ItemsInput = z.infer<typeof itemsInputSchema>
export type ItemsUpdate = z.infer<typeof itemsUpdateSchema>

export const itemsSchema = {
    model: itemsModelSchema,
    dto: itemsDtoSchema,
    input: itemsInputSchema,
    update: itemsUpdateSchema,
}

export default itemsTable
