import { relations, sql } from "drizzle-orm";
import { boolean, integer, pgTable, serial, timestamp, varchar } from "drizzle-orm/pg-core";
import userTable from "./user.schema";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";

const passwordRecoveryTable = pgTable('reset_tickets',{
  id: serial('id').primaryKey()
  .unique()
  .notNull(),
  userId: integer('user_id')
    .references(() => userTable.id, {onDelete: 'cascade'})
    .notNull(),
  tokenHash: varchar('token_hash', {length: 255})
    .notNull(),
  expirationDate: timestamp('expiration_date', { mode: 'string'})
    .notNull(),
  tokenUsed: boolean('token_used')
    .notNull()
    .default(false),
  validToken: boolean('valid_token')
    .notNull()
    .default(true),
  createdAt: timestamp('created_at', {
    mode: 'string',
  })
    .notNull()
    .defaultNow(),
  updatedAt: timestamp('updated_at', {
    mode: 'string',
  })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),

})

export const passwordRecoveryTableRelations = relations(
  passwordRecoveryTable,
  ({ one }) => ({
    user: one(userTable, {
      fields: [passwordRecoveryTable.userId],
      references: [userTable.id]
    })   
  })
)

const passwordRecoveryModelSchema = createSelectSchema(passwordRecoveryTable)
const passwordRecoveryDtoSchema = passwordRecoveryModelSchema.omit({
  tokenHash: true
})
const passwordRecoveryInputSchema = createInsertSchema(passwordRecoveryTable)
const passwordRecoveryUpdateSchema = passwordRecoveryInputSchema
  .partial()
  .required({ id: true })

export type PasswordRecoveryModel = z.infer<typeof passwordRecoveryModelSchema>
export type PasswordRecoveryDto = z.infer<typeof passwordRecoveryDtoSchema>
export type PasswordRecoveryInput = z.infer<typeof passwordRecoveryInputSchema>
export type PasswordRecoveryUpdate = z.infer<typeof passwordRecoveryUpdateSchema>

export const passwordRecoverySchemas = {
  model: passwordRecoveryModelSchema,
  dto: passwordRecoveryDtoSchema,
  input: passwordRecoveryInputSchema,
  update: passwordRecoveryInputSchema
}

export default passwordRecoveryTable