import { boolean, integer, pgTable, serial, text, timestamp } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";
import resourceTable from "./resource.schema";
import userTable from "./user.schema";
import { sql } from "drizzle-orm";

const submissionTable = pgTable('submissions', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  is_accepted: boolean('is_accepted')
    .default(false)
    .notNull(),
  justification: text('justification'),
  resource_id: integer('resource_id')
    .notNull()
    .references(() => resourceTable.id, {onDelete: 'cascade'}),
  submitter_id: integer('submitter_id')
    .notNull()
    .references(() => userTable.id, {onDelete: 'cascade'}),
  curator_id: integer('curator_id')
    .references(() => userTable.id, {onDelete: 'cascade'}),
  created_at: timestamp('created_at')
    .notNull()
    .defaultNow(),
  updated_at: timestamp('updated_at')
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),
  answered_at: timestamp('answered_at'),
  q1: boolean('q1'),
  q2: boolean('q2'),
  q3: boolean('q3'),
  q4: boolean('q4'),
})

const submissionModelSchema = createSelectSchema(submissionTable)
const submissionDtoSchema = submissionModelSchema
const submissionInputSchema = createInsertSchema(submissionTable)
const submissionUpdateSchema = submissionInputSchema.partial().required({ id: true, resource_id: true})

export type SubmissionModel = z.infer<typeof submissionModelSchema>
export type SubmissionDto = z.infer<typeof submissionDtoSchema>
export type SubmissionInput = z.infer<typeof submissionInputSchema>
export type SubmissionUpdate = z.infer<typeof submissionUpdateSchema>

export const submissionSchemas = {
  submissionModelSchema,
  submissionDtoSchema,
  submissionInputSchema,
  submissionUpdateSchema,
}

export default submissionTable