import { resourceSchema } from "@/db/schema/resource.schema";
import { ResourceService } from "@/services/resource.service";
import { zValidator } from "@hono/zod-validator";
import { honoWithJwt } from '..';
import Container from "typedi";
import { createApexError, HttpStatus } from "@/services/error.service";
import { Hono } from "hono";
import { resourceStatsSchema } from "@/db/schema/resource-stats.schema";
import { ResourceStatsService } from "@/services/resource-stats.service";

const service = Container.get(ResourceService);
const serviceStats = Container.get(ResourceStatsService);

export const resourceRouter = honoWithJwt()
    // rota para criar um recurso
    .post('/create',
        zValidator('json', resourceSchema.input),

        async (c) => {
            try {
                const input = await c.req.valid('json')
                //cria o stats do recurso correspondente
                const stats = resourceStatsSchema.dto.parse(
                    await serviceStats.create()
                )

                input.resource_stats_id = stats.id
                const resource = resourceSchema.dto.parse(
                    await service.create(input)
                )

                return c.json({ resource, stats })
            } catch (e) {
                return c.json(
                    createApexError({
                        status: 'error',
                        message: 'could not create the resource',
                        code: HttpStatus.BAD_REQUEST,
                        path: c.req.routePath,
                        suggestion: 'check the json input and try again',
                    }),
                    HttpStatus.BAD_REQUEST
                )
            }

        }
    )

    // rota para atualizar um recurso    
    .post('/update',
        zValidator('json', resourceSchema.update),
        async (c) => {
            try {
                const input = await c.req.valid('json')
                const resource = resourceSchema.dto.parse(await service.update(input))

                return c.json({ resource })
            } catch (e) {
                return c.json(
                    createApexError({
                        status: 'error',
                        message: 'could not update the resource',
                        code: HttpStatus.BAD_REQUEST,
                        path: c.req.routePath,
                        suggestion: 'check the json input and try again',
                    }),
                    HttpStatus.BAD_REQUEST
                )
            }
        })

    // rota para deletar um recurso        
    .post('/deleteData/:id',
        async (c) => {
            try {
                const id = +c.req.param('id')
                const resource = resourceSchema.dto.parse(await service.deleteData(id))

                //onDelete: "cascade" -> presente no stats-resources.schema.ts ja exclui o stats 

                return c.json(resource)
            } catch (e) {
                return c.json(
                    createApexError({
                        status: 'error',
                        message: 'could not delete the resource',
                        code: HttpStatus.BAD_REQUEST,
                        path: c.req.routePath,
                        suggestion: 'check the id and try again',
                    }),
                    HttpStatus.BAD_REQUEST
                )
            }
        }
    )

    //rota para dizer que o recurso foi deletado
    .post('/delete/:id',
        async (c) => {
            try {
                const id = +c.req.param('id')
                const resource = resourceSchema.dto.parse(await service.delete(id))

                return c.json(resource)
            } catch (e) {
                return c.json(
                    createApexError({
                        status: 'error',
                        message: 'could not delete the resource',
                        code: HttpStatus.BAD_REQUEST,
                        path: c.req.routePath,
                        suggestion: 'check the id and try again',
                    }),
                    HttpStatus.BAD_REQUEST
                )
            }
        }
    )

    //rota para tornar o recurso ativo
    .post('/active/:id',
        async (c) => {
            try {
                const id = +c.req.param('id')
                const resource = resourceSchema.dto.parse(await service.active(id))

                return c.json(resource)
            } catch (e) {
                return c.json(
                    createApexError({
                        status: 'error',
                        message: 'could not active the resource',
                        code: HttpStatus.BAD_REQUEST,
                        path: c.req.routePath,
                        suggestion: 'check the id and try again',
                    }),
                    HttpStatus.BAD_REQUEST
                )
            }
        }
    )


export const publicResourceRouter = new Hono()
// rota para listar todos os recursos    
    .get('/all', async (c) => {
        try {
            const resource = resourceSchema.dto.array().parse(await service.findMany())
            return c.json({ resource })
        } catch (e) {
            return c.json(
                createApexError({
                    status: 'error',
                    message: 'could not find any resource',
                    code: HttpStatus.NOT_FOUND,
                    path: c.req.routePath,
                    suggestion: 'are you sure there are resource?',
                }),
                HttpStatus.NOT_FOUND
            )
        }
    }
    )

// rota para pegar um recurso especifico    
    .get('/:id',
        async (c) => {
            try {
                const id = +c.req.param('id')
                const resource = resourceSchema.dto.parse(await service.findById(id))

                return c.json(resource)
            } catch (e) {
                return c.json(
                    createApexError({
                        status: 'error',
                        message: 'could not find the resource',
                        code: HttpStatus.BAD_REQUEST,
                        path: c.req.routePath,
                        suggestion: 'are you sure the resource exists?',

                    }),
                    HttpStatus.BAD_REQUEST
                )
            }
        }
    )
